require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::ApplicantsController, :type => :api do

  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  include JwtAuthHelper
  let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}

  check_mutate_params(
    model: Hermes::Applicant, 
    mutate_params: Hermes::Api::Dmo::ApplicantsController.mutate_params, 
    factory: :applicant
  )

  describe 'with project oriented roles' do
    let!(:demandeur) { create(:user, auth_roles: ['demandeur']) }

    let!(:bailleur) { create(:user, auth_roles: ['bailleur']) }
    let!(:entreprise) { create(:user, auth_roles: ['entreprise']) }
    let!(:dc_channel) {create(:housing_channel, key: 'dc')}

    describe 'POST as entreprise with main_user_id' do
      before :each do
        authenticated_header(request, demandeur)
        post 'create', {
          data: {
            attributes: {
              main_user_id: demandeur.id.to_s
            }
          }
        }
      end

      it 'returns status code 201' do
        expect(response).to have_http_status(201)
      end

      it 'returns valid json' do
        attributes = json['data']['attributes']
        expect(attributes['main_user_id']).to eq demandeur.id.to_s
      end
    end

  end

  describe 'normal' do
    let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

    describe 'POST /applicants' do
      describe 'applicant created with identity' do
        before :each do
          authenticated_header(request, user)
          post 'create', {
            data: {
              attributes: {
                first_name: 'Joseph',
                last_name: 'Joestar',
                email: 'josephjoestar@test.fr'
              }
            }
          }
        end

        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end

        it 'returns valid JSON' do
          attributes = json['data']['attributes']
          expect(attributes['full_name']).to eq 'Joseph Joestar'
          expect(attributes['email']).to eq 'josephjoestar@test.fr'
        end
      end

      describe 'Create applicant with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
            data: {
              attributes: {
                main_user_id: responsible.id,
              }
            }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end

        describe 'then same user creates another applicant' do
          before :each do
            authenticated_header(request, responsible)
            post 'create', {
              data: {
                attributes: {
                  first_name: 'Janine',
                  last_name: 'Joseph',
                  email: 'janinejoseph@test.fr'
                }
              }
            }
          end

          it 'returns status code 201' do
            expect(response).to have_http_status(201)
          end

          it 'returns valid JSON' do
            attributes = json['data']['attributes']
            expect(attributes['full_name']).to eq 'Janine Joseph'
            expect(attributes['email']).to eq 'janinejoseph@test.fr'
          end
        end
      end
    end

    describe 'GET /applicants' do
      let!(:applicant1) {create(:applicant, nationality: 'belge', apl_amount: 50)}
      let!(:applicant3) {create(:applicant, nationality: 'quebecquoise', apl_amount: 150)}
      let!(:applicant2) {create(:applicant, nationality: 'francaise', apl_amount: 150)}

      [
        {user: 'responsible', total_objects: 3},
        {user: 'superadministrateur', total_objects: 3},
        {user: 'user', total_objects: 0},
      ].each do |a|
        describe "as #{a[:user]}" do

          before :each do
            authenticated_header(request, eval(a[:user]))
            get 'index'
          end
          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end
          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to eq(a[:total_objects])
            expect(json['meta']['pagination']).to be_present
            expect(json['meta']['pagination']['total_objects']).to eq(a[:total_objects])
          end
        end
      end

      Hermes::Api::Dmo::ApplicantsController.acceptable_params.each do |acceptable_param, options|
        type = options[:type]
        next if ['housing_request_ids'].include?(acceptable_param.to_s)
        if type
          describe "with sort on #{acceptable_param}" do
            before :each do
              (1..3).each do |i|
                record = eval("applicant#{i}")
                record.unset(acceptable_param.to_sym)
                record.set(acceptable_param.to_sym => get_typed_value(type, (i+1) % 3))
              end 

              authenticated_header(request, responsible)
              get 'index', {sort: {acceptable_param => 1}}
            end

            it 'returns sorted results' do
              applicant_attribute = if type == 'DateTime'
                json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param.to_s])}
              elsif type == 'Date'
                json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param.to_s])}
              else
                json['data'].map {|unit| unit['attributes'][acceptable_param.to_s]}
              end
              expect(response.status).to eq(200)
              expect(applicant_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
            end
          end
        end
      end

      describe 'GET /applicants with operators' do
        let!(:applicant1) {create(:applicant, nationality: 'belge', apl_amount: 50)}
        let!(:applicant3) {create(:applicant, nationality: 'quebecquoise', apl_amount: 150)}
        let!(:applicant2) {create(:applicant, nationality: 'Samoan', apl_amount: 150)}
        describe 'with $lt' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {apl_amount: {'$lt': 100}}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $in' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {nationality: {'$in': ['belge', 'flamand']}}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $ne' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {apl_amount: {'$ne': 50}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(2)
          end
        end
        describe 'with $regex' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {nationality: {'$regex': /^S/}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
      end
    end

    describe 'GET /applicants/:id' do
      describe 'verif it work' do
        let!(:applicant1) {create(:applicant, nationality: 'belge', apl_amount: 50)}

        # it_behaves_like 'true user only collaborator allowed' do
        #   let(:applicant) { applicant1 }
        #   let(:path) { ['show', { id: applicant1.id }] }
        # end

        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: applicant1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
      describe 'verif it work not work with a non authorized user' do
        let!(:applicant1) {create(:applicant, nationality: 'belge', apl_amount: 50)}

        before :each do
          authenticated_header(request, user)
          get 'show', {id: applicant1.id}
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end

      describe 'verif it work  for superadministrateur' do
        let!(:applicant1) {create(:applicant, nationality: 'belge', apl_amount: 50)}

        before :each do
          authenticated_header(request, superadministrateur)
          get 'show', {id: applicant1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
      end
    end

    describe 'PATCH /applicant/:id for year_minus_two' do
      let!(:demandeur_test) { create(:user, auth_roles: ['demandeur']) }
      let!(:applicant_test) do
        create(
          :applicant,
          amount_year_minus_two: 1200,
          main_user_id: demandeur_test.id,
          user_id: demandeur_test.id,
          monthly_incomes: {
            'SAL': 1234
          }
        )
      end

      let!(:request_test) do
        create(
          :housing_request,
          is_active: true,
          main_applicant: applicant_test
        )
      end

      let(:params) { { amount_year_minus_two: 1201.0 } }

      before :each do
        authenticated_header(request, demandeur_test)
        patch 'update', {
          id: applicant_test.id.to_s,
          data: {
            attributes: params
          }
        }
      end

      it 'returns 200' do
        expect(response).to have_http_status(200)
      end

      it 'returns valid json' do
        data = json['data']
        attributes = data['attributes']
        expect(data['id']).to eq applicant_test.id.to_s
        expect(attributes['amount_year_minus_two']).to be params[:amount_year_minus_two]
      end

      describe 'GET same object' do
        before :each do
          authenticated_header(request, demandeur_test)
          get 'show', id: applicant_test.id.to_s
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          attributes = data['attributes']
          expect(data['id']).to eq applicant_test.id.to_s
          expect(attributes['amount_year_minus_two']).to be params[:amount_year_minus_two]
        end
      end

      describe 'update_forbidden if application but allowed for admins' do
        let!(:housing_offer1) {create(:housing_offer)}
        let!(:housing_application2) {create(:housing_application, housing_request: request_test, housing_offer: housing_offer1)}
        let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}

        before :each do
          authenticated_header(request, responsible)
          patch 'update', {
            id: applicant_test.id.to_s,
            data: {
              attributes: { amount_year_minus_two: 11.0 }
            }
          }
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

      end
    end

    describe 'PATCH /applicants/:id' do
      applicant_params = {}
      Hermes::Api::Dmo::ApplicantsController.mutate_params.each do |param, options|
        applicant_params[param] = get_typed_value(options[:type])
      end
      let!(:applicant) {create(:applicant, **applicant_params)}

      describe 'put' do
        let!(:applicant3) {create(:applicant, auto_set_user: false, first_name: 'Jac', last_name: "Mike", email: "jackie.mike@gmail.com", writers:[responsible.id], status: "OK", monthly_incomes: {porn: 30}, monthly_charges: {cat_food: 10})}
        before :each do
          authenticated_header(request, responsible)
          patch 'update', {
            id: applicant3.id.to_s,
            data: {
              attributes: {
                sms_notification: true
              }
            }
          }
        end
        it 'returns status code 200 and the value appears in json' do
          expect(response).to have_http_status(200)
          expect(json['data']['attributes']['first_name']).to eq("Jac")
        end
      end


      describe 'patch applicant entity_id as applicant' do
        let!(:entity) {create(:entity, siret: '00000000000001')}
        let!(:entity2) {create(:entity, siret: '00000000000002', auto_request_validation: false)}
        let!(:user1){create(:user, auth_roles: ["demandeur"])}
        let!(:reader_entity_1){create(:user, group_ids: [entity.manager_group_id])}
        let!(:reader_entity_2){create(:user, group_ids: [entity2.manager_group_id])}
        let!(:applicant){create(:applicant, user: user1 )}
        let!(:hr) {create(:housing_request, main_applicant: applicant, entity_validated: true, is_active: true, user: user1)}
        before :each do
          authenticated_header(request, user1)
          patch 'update', {
            id: applicant.id.to_s,
            data: {
              attributes: {
                main_employer_siret: entity2.siret,
              }
            }
          }
        end
        it 'returns status code 200 and the value appears in json' do
          expect(response).to have_http_status(200)
          expect(applicant.reload.readable_by?(user1)).to eq true
          expect(hr.reload.entity).to eq(entity2)
          expect(hr.reload.entity_validated).to eq(false)
          expect(hr.reload.readable_by?(user1)).to eq true
          expect(hr.reload.readable_by?(reader_entity_1)).to eq false
          expect(hr.reload.readable_by?(reader_entity_2)).to eq true
        end
      end
    end

    describe 'DELETE /applicants/:id' do
      let!(:applicant) {create(:applicant, user_id: responsible.id.to_s)}
      before :each do
        authenticated_header(request, responsible)

        delete 'destroy', {
            id: applicant.id.to_s
        }
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(204)
      end
    end
  end

  describe 'GET /applicants/find_by_nud' do
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:applicant) { create(:applicant) }
    let!(:housing_request) { create(:housing_request, main_applicant_id: applicant.id) }
    let!(:nud) { create(:nud, unique_number: "111101657788975B02", applicant: applicant, housing_request: housing_request) }
    before :each do
      authenticated_header(request, responsible)
    end

    it 'return existing applicant' do
      get :find_by_nud, nud: nud.unique_number
      expect(response).to have_http_status(200)
      expect(json['data']['id']).to eq applicant.id.to_s
    end

    it 'return created applicant' do
      get :find_by_nud, nud: '1110121021518G1104', birth_date: '1982-10-10'
      expect(response).to have_http_status(200)
      expect(json['data']['id']).not_to be_nil
      expect(json['data']['id']).not_to eq applicant.id.to_s
    end

    it 'return invalid birth_date error' do
      get :find_by_nud, nud: '1110121021518G1104', birth_date: '1992-05-10'
      expect(response).to have_http_status(422)
      expect(json['errors'].first['code']).to include('invalid_birth_date')
    end

    it 'return not found for invalid nud' do
      get :find_by_nud, nud: '000000000000000000', birth_date: '1991-05-10'
      expect(response).to have_http_status(404)
      expect(json['errors'].first['code']).to include('invalid_nud')
    end
  end


  describe 'update reference_tax_revenues on hr' do 
    let!(:superadministrateur) { create(:user, auth_roles: ['superadministrateur']) }
    let!(:applicant) {create(:applicant, amount_year_minus_two: 100)}
    let!(:coapplicant) {create(:applicant, amount_year_minus_two: 100)}
    let!(:coapplicant2) {create(:applicant, amount_year_minus_two: 100)}
    let!(:hr) {create(:housing_request, main_applicant: applicant, coapplicants: [{id: coapplicant.id.to_s, rel_type: 'C'}, {id: coapplicant2.id.to_s, rel_type: 'C'}], is_active: true)}

    it 'coapplicant' do
      authenticated_header(request, superadministrateur)
      patch 'update', {
        id: coapplicant.id.to_s,
        data: {
          attributes: { amount_year_minus_two: 5000}
        }
      }

      expect(response).to have_http_status 200
      expect(hr.reload.reference_tax_revenues).to eq(5200)
    end

  end

  describe 'POST /switch' do
    let!(:reader) { create :user, roles: ['Superadmin', 'Administrator'] }
    let!(:entreprise) { create :user, auth_roles: ['entreprise'] }
    let!(:target) { create :user, main_applicant_id: applicant.id }
    let!(:entity) { create :entity, readers: [entreprise.id.to_s], siret: '12345678901234' }
    let!(:form_level) { create :entity_form_level, entity_id: entity.id }
    let!(:applicant) { create :applicant, main_employer_siret: entity.siret, auto_set_user: false }
    let!(:unkown_applicant) { create :applicant, main_employer_siret: entity.siret, auto_set_user: false }

    it 'returns unauthorized error' do
      authenticated_header(request, reader)
      post 'switch', { id: applicant.id }

      expect(response).to have_http_status 401
      expect(json['message']).to eq 'Votre role doit être \'entreprise\' pour effectuer cette action.'
    end

    context 'as entreprise' do
      let!(:housing_request2) { create :housing_request, main_applicant_id: applicant.id, is_active: false}
      let!(:housing_request) { create :active_request, main_applicant_id: applicant.id }
      before do
        authenticated_header(request, entreprise)
      end

      it 'returns not_found error' do
        post 'switch', { id: unkown_applicant.id }
        expect(response).to have_http_status 404
        expect(json['message']).to eq 'Utilisateur target non trouvé.'
      end

      it 'requires entity_id param' do
        post 'switch', { id: applicant.id }
        expect(response).to have_http_status 406
        expect(json['errors'][0]['message']).to eq 'param is missing or the value is empty: entity_id'
      end

      it 'returns not_found error' do
        post 'switch', { id: applicant.id, entity_id: '123' }
        expect(response).to have_http_status 404
        expect(json['success']).to eq false
        expect(json['message']).to eq 'Entity not found'
      end

      it 'returns not_found error' do
        applicant.first_active_request.update(is_active: false)
        post 'switch', { id: applicant.id,  entity_id: entity.id }
        expect(response).to have_http_status 404
        expect(json['success']).to eq false
        expect(json['message']).to eq 'No active housing_request for this applicant'
      end

      context 'when no entity_form' do
        it 'returns not_found error' do
          post 'switch', { id: applicant.id, entity_id: entity.id }
          expect(response).to have_http_status 404
          expect(json['success']).to eq false
          expect(json['message']).to eq 'delegation not found'
        end
      end

      context 'when entity_form without delegation and/or delegation signed' do
        let!(:entity_form2) do
          create(
            :entity_form,
            housing_request_id: housing_request2.id,
            entity_form_level_id: form_level.id,
            delegation_signed: false
          )
        end
        let!(:entity_form) do
          create(
            :entity_form,
            housing_request_id: housing_request.id,
            entity_form_level_id: form_level.id,
            delegation_signed: false
          )
        end
        
        it 'returns unauthorized error' do
          entity_form2.set(applicant_id: applicant.id)
          entity_form.set(applicant_id: applicant.id)
        
          post 'switch', { id: applicant.id, entity_id: entity.id }
          expect(response).to have_http_status 401
          expect(json['success']).to eq false
          expect(json['message']).to eq 'delegation not available or not signed'
        end
      end

      context 'when entity_form with delegation and delegation signed' do
        let!(:form_level) { create :entity_form_level, entity_id: entity.id, delegation: true, level: 4 }
        let!(:entity_form2) do
          create(
            :entity_form,
            housing_request_id: housing_request2.id,
            entity_form_level_id: form_level.id,
          )
        end
        let!(:entity_form) do
          create(
            :entity_form,
            housing_request_id: housing_request.id,
            entity_form_level_id: form_level.id,
          )
        end
      
        before do
          entity_form2.set(applicant_id: applicant.id)
          entity_form.set(applicant_id: applicant.id)
          post 'switch', { id: applicant.id, entity_id: entity.id }
        end
        it 'succeeds' do
          expect(response).to have_http_status 200
        end
        it 'returns target infos' do
          expect(json["email"]).to eq target.email
          expect(json['jwt_token']).not_to be_empty
        end
      end
        context 'find by email' do
          let!(:target_by_email) { create :user, email: 'test@test.com' }
          let!(:applicant_by_email) { create :applicant, main_employer_siret: entity.siret, email: 'test@test.com', auto_set_user: false }
          let!(:housing_request) { create :active_request, main_applicant_id: applicant_by_email.id }
          
          let!(:form_level) { create :entity_form_level, entity_id: entity.id, delegation: true, level: 4 }
          let!(:entity_form2) do
            create(
              :entity_form,
              housing_request_id: housing_request2.id,
              entity_form_level_id: form_level.id,
            )
          end
          let!(:entity_form) do
            create(
              :entity_form,
              housing_request_id: housing_request.id,
              entity_form_level_id: form_level.id,
            )
          end
          
          before do
            post 'switch', { id: applicant_by_email.id, entity_id: entity.id }
          end
          it 'succeeds' do
            expect(response).to have_http_status 200
          end
          it 'returns target infos' do
            expect(json['email']).to eq target_by_email.email
            expect(json['jwt_token']).not_to be_empty
          end
        
      end
    end
  end
end

describe Hermes::Api::Dmo::ApplicantsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  require_relative '../../swagger_helper'
  tags = 'API_DMO_APPLICANTS'
  include JwtAuthHelper
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:admin) {create(:administrator)}

  context 'swagger' do 
    path '/api/dmo/applicants' do
      get 'Retrieves all applicants' do
        tags tags
        produces 'application/json'

        Hermes::Api::Dmo::ApplicantsController.acceptable_params.sort.each do |params_name, options|
          parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
        end

        security [Bearer: {}]
        paginator

        response '401', 'unauthorized' do
          let(:Authorization) { "Bearer" }

          run_test!
        end 

        response '200', 'applicants found' do
          let(:Authorization) { "Bearer #{token_for(admin)}" }
          let!(:applicants) { create_list(:applicant, 5,user_id: admin.id.to_s) }

          include_context 'with integration test'
          include_context 'valid jsonapi collection' do
            let(:expected_collection) { applicants }
          end
        end
      end
    end

    path '/api/dmo/applicants/find_by_nud' do
      get 'Retrieves or create applicant with nud' do
        tags tags
        produces 'application/json'

        parameter name: :nud, in: :query, type: :string, required: true
        parameter name: :birth_date, in: :query, type: :string, required: false, description: "Doit être renseigné si le demandeur n'existe pas. Format attendu : YYYY-MM-DD"
        security [Bearer: {}]

        response '200', 'applicant found' do
          let(:Authorization) { "Bearer #{token_for(admin)}" }
          let!(:applicant) { create(:applicant) }
          let!(:housing_request) { create(:housing_request, main_applicant_id: applicant.id) }
          let!(:nud_inst) { create(:nud, unique_number: "111101657788975B02", applicant: applicant, housing_request: housing_request) }
          let!(:nud) { nud_inst.unique_number }
          include_context 'with integration test'
          include_context 'valid jsonapi ressource' do
            let(:expected_ressource) { applicant }
          end
        end

        response '200', 'applicant found' do
          let(:Authorization) { "Bearer #{token_for(admin)}" }
          let!(:nud) { '1110121021518G1104' }
          let!(:birth_date) { '1982-10-10' }
          include_context 'with integration test'
          run_test!
        end

        response '404', 'invalid nud' do
          let(:Authorization) { "Bearer #{token_for(admin)}" }
          let!(:nud) { '000000000000000000' }
          let!(:birth_date) { '1992-05-10' }
          run_test!
        end

        response '422', 'invalid birth_date' do
          let(:Authorization) { "Bearer #{token_for(admin)}" }
          let!(:nud) { '1110121021518G1104' }
          let!(:birth_date) { '1992-05-10' }
          run_test!
        end
      end
    end

    path '/api/dmo/applicants' do
      post 'Create applicant' do
        tags tags
        produces 'application/json'
        consumes 'application/json'
        security [Bearer: {}]
        json_params = {}
        Hermes::Api::Dmo::ApplicantsController.mutate_params.each do |key, options|
          json_params[key] = nice_type(options[:type])
        end
        parameter(
          name: :params,
          in: :body,
          schema: jsonify_params(json_params),
        )
        
        response '401', 'unauthorized' do
          let(:Authorization) { "Bearer" }
          let(:params) { {} }

          run_test!
        end 

        response '201', 'applicant created' do
          let(:Authorization) {"Bearer #{token_for(admin)}"}
          let(:params) do
            jsonapi_attributes(
                user_id: admin.id.to_s
            )
          end
          include_context 'with integration test'
          include_context 'valid jsonapi'
          include_context 'not containings errors'

          after do |_example|
            json = JSON.parse(response.body)

            expect(json['data']).to be_a Hash
            expect(json['data']['id']).to_not be nil
            expect(json['data']['type']).to eq 'applicants'
          end
        end
      end
    end

    path '/api/dmo/applicants/{id}' do
      get 'Retrieves a single applicant by id' do
        tags tags
        produces 'application/json'
        parameter name: :id, in: :path, type: :string
        security [Bearer: {}]

        response '401', 'unauthorized' do
          let(:Authorization) { "Bearer" }
          let(:id) { 1 }

          run_test!
        end 

        response '404', 'not found' do
          let(:Authorization) { "Bearer #{token_for(admin)}" }
          let(:id) { 1 }

          run_test!
        end 

        response '200', 'applicant found' do
          let(:Authorization) { "Bearer #{token_for(admin)}" }
          let(:applicant) { create(:applicant) }
          let(:id) { applicant.id }

          include_context 'with integration test'
          include_context 'valid jsonapi ressource' do
            let(:expected_ressource) { applicant }
          end
        end

      end
    end

    path '/api/dmo/applicants/{id}' do
      put 'edit applicant' do
        tags tags
        produces 'application/json'
        consumes 'application/json'
        parameter name: :id, in: :path, type: :string
        security [Bearer: {}]
        json_params = {}
        Hermes::Api::Dmo::ApplicantsController.mutate_params.each do |key, options|
          json_params[key] = nice_type(options[:type])
        end
        parameter(
          name: :params,
          in: :body,
          schema: jsonify_params(json_params),
        )

        response '401', 'unauthorized' do
          let(:Authorization) { "Bearer" }
          let(:id) { 1 }
          let(:params) do
            jsonapi_attributes(
                id: id,
                first_name: 'Joseph',
                last_name: 'Joestar'
            )
          end

          run_test!
        end 

        response '404', 'not found' do
          let(:Authorization) {"Bearer #{token_for(admin)}"}
          let(:id) { 1 }
          let(:params) do
            jsonapi_attributes(
                id: id,
                first_name: 'Joseph',
                last_name: 'Joestar'
            )
          end

          run_test!
        end 

        response '200', 'applicant updated' do
          let(:Authorization) {"Bearer #{token_for(admin)}"}
          let(:applicant) {create(:applicant, user_id: admin.id.to_s)}
          let(:id) {applicant.id}
          let(:params) do
            jsonapi_attributes(
                id: id,
                first_name: 'Joseph',
                last_name: 'Joestar'
            )
          end
          include_context 'with integration test'
          include_context 'valid jsonapi'
          include_context 'not containings errors'

          after do |_example|
            json = JSON.parse(response.body)

            data = json['data']
            expect(data).to be_a Hash
            expect(data['id']).to_not be nil
            attributes = data['attributes']
            expect(attributes['first_name']).to eq 'Joseph'
            expect(attributes['last_name']).to eq 'Joestar'
          end
        end
      end
    end

    path '/api/dmo/applicants/{id}' do
      delete 'Delete a single applicant by id' do
        tags tags
        produces 'application/json'
        parameter name: :id, in: :path, type: :string
        security [Bearer: {}]

        response '401', 'unauthorized' do
          let(:Authorization) { "Bearer" }
          let(:id) { 1 }

          run_test!
        end 

        response '404', 'not found' do
          let(:Authorization) { "Bearer #{token_for(admin)}" }
          let(:id) { 1 }

          run_test!
        end 


        response '204', 'applicant deleted' do
          let(:Authorization) { "Bearer #{token_for(admin)}" }
          let(:id) { create(:applicant, responsible: admin.id).id }
          run_test!
        end
      end
    end

    path '/api/dmo/applicants/{id}/switch' do
      post 'Switch to the applicant' do
        tags tags
        parameter name: :id, in: :path, type: :string
        parameter name: :entity_id, in: :query, type: :string, required: true 
        produces 'application/json'
        consumes 'application/json'
        security [Bearer: {}]

        let!(:entreprise) { create :user, auth_roles: ['entreprise'] }
        let!(:entity) { create :entity, readers: [entreprise.id.to_s], siret: '12345678901234' }
        let!(:applicant) { create :applicant, main_employer_siret: entity.siret, auto_set_user: false }


        response '401', 'unauthorized' do
          let(:Authorization) { "Bearer" }
          let(:id) { applicant.id }
          let(:entity_id) { entity.id }
          run_test!
        end 

        response '404', 'not found' do
          let(:Authorization) { "Bearer #{token_for(entreprise)}" }
          let(:id) { 1 }
          let(:entity_id) { entity.id }

          run_test!
        end 

        response '406', 'missing params' do
          let(:Authorization) { "Bearer #{token_for(entreprise)}" }
          let!(:applicant) { create :applicant }
          let(:id) { applicant.id }
          let(:entity_id) { nil }

          run_test!
        end 

        response '200', 'Swtich to applicant' do
          let(:Authorization) {"Bearer #{token_for(entreprise)}"}
          let!(:user) { create :user, email: 'test@test.com', main_applicant_id: applicant.id }
          let!(:housing_request) { create :active_request, main_applicant_id: applicant.id }
          let!(:form_level) { create :entity_form_level, entity_id: entity.id, delegation: true, level: 4 }
          let!(:entity_form) do
            create(
              :entity_form,
              housing_request_id: housing_request.id,
              entity_form_level_id: form_level.id,
            )
          end
          let(:id) { applicant.id }
          let(:entity_id) { entity.id }

          include_context 'with integration test'
          include_context 'not containings errors'

          after do |_example|
            json = JSON.parse(response.body)

            expect(json['email']).to eq 'test@test.com'
            expect(json["jwt_token"]).not_to be_empty
          end
        end
      end
    end
  end
end