require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::HousingChannelsController, :type => :api do
  routes { Hermes::Engine.routes }
  include Devise::Test::ControllerHelpers
  include JwtAuthHelper

  check_mutate_params(
    model: Hermes::HousingChannel, 
    mutate_params: Hermes::Api::Dmo::HousingChannelsController.mutate_params, 
    factory: :housing_channel,
    user_roles: ['Administrator'],
    min_params: {
      key: 'test'
    }
  )

  let!(:responsible) { create(:user, roles: ['Administrator']) }
  let!(:roles) { %w[bailleur entreprise demandeur Manager Supervisor] }
  let!(:user) { create(:user, roles: roles.sample(rand(1..3))) }

  let(:channel) { create :housing_channel, key: 'dalo' }
  let(:params) do
    { attributes: { key: 'priority_public', label: 'Public prioritaire' } }
  end

  before { authenticated_header(request, responsible) }

  describe 'GET index' do
    let!(:channels) { create_list :housing_channel, 5 }
    subject { get 'index' }

    it 'returns status code 200' do
      subject
      expect(response).to have_http_status 200
    end

    it 'returns a valid json' do
      subject
      expect(json['data'].length).to eq 5
      expect(json['meta']['pagination']).to be_present
      expect(json['meta']['pagination']['total_objects']).to eq 5
    end

    it 'works as normal user' do 
      authenticated_header(request, user) 
      subject
      expect(response).to have_http_status 200
      expect(json['data'].length).to eq 5

    end 
  end

  describe 'GET show' do
    subject { get 'show', { id: channel.id } }

    it 'returns status code 200' do
      subject
      expect(response).to have_http_status 200
    end

    it 'returns a valid json' do
      subject
      expect(json['data']['id']).to eq channel.id.to_s
      expect(json['data']['attributes']['key']).to eq 'dalo'
    end

    it 'works as normal user' do 
      authenticated_header(request, user) 
      subject
      expect(response).to have_http_status 200
    end 
  end

  describe 'POST create' do
    subject { post 'create', { data: params } }

    it_behaves_like 'admin only'

    it 'succeeds' do
      subject
      expect(response).to have_http_status 201
    end

    it 'creates a housing channel' do
      expect do
        post 'create', { data: params }
      end.to change { Hermes::HousingChannel.count }.by 1
    end

    it 'returns a valid json' do
      subject
      expect(json['data']['attributes']['key']).to eq 'priority_public'
      expect(json['data']['attributes']['label']).to eq 'Public prioritaire'
    end
  end

  describe 'PATCH update' do
    subject { patch 'update', { id: channel.id, data: params } }

    it_behaves_like 'admin only'

    it 'succeeds' do
      subject
      expect(response).to have_http_status 200
    end

    it 'updates the channel' do
      expect do
        subject
      end.to change { channel.reload.key }.from('dalo').to 'priority_public'
    end

    it 'returns a valid json' do
      subject
      expect(json['data']['id']).to eq channel.id.to_s
      expect(json['data']['attributes']['key']).to eq 'priority_public'
      expect(json['data']['attributes']['label']).to eq 'Public prioritaire'
    end
  end

  describe 'DELETE destroy' do
    let(:rand_user) { create :user }
    let!(:owned_channel) { create :housing_channel, user_id: responsible.id }

    subject { delete 'destroy', { id: owned_channel.id.to_s } }

    it_behaves_like 'admin only'

    context 'for owner' do
      it 'succeeds' do
        subject
        expect(response).to have_http_status 204
      end

      it 'destroys the channel' do
        expect do
          subject
        end.to change { Hermes::HousingChannel.count }.by -1
      end
    end

    context 'for another user' do
      before { authenticated_header(request, rand_user) }

      it 'returns error' do
        subject
        expect(response).to have_http_status 401
      end
    end
  end

end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::HousingChannelsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_HOUSING_CHANNELS'
  include JwtAuthHelper
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) { create(:user, roles: ['Administrator']) }
  let!(:admin) { create(:administrator) }

  path '/api/dmo/housing_channels' do
    get 'Retrieves all housing channels' do
      tags tags
      produces 'application/json'

      Hermes::Api::Dmo::HousingChannelsController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'housing channels found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:housing_channels) { create_list(:housing_channel, 5) }

        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) { housing_channels }
        end
      end
    end
  end

  path '/api/dmo/housing_channels/{id}' do
    get 'Retrieves a single housing channel by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      let(:housing_channel) { create(:housing_channel) }
      let(:id) { housing_channel.id }

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }
        run_test!
      end 

      response '200', 'housing channel found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) { housing_channel }
        end
      end
    end
  end

  path '/api/dmo/housing_channels' do
    post 'Create housing channel' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::HousingChannelsController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )

      let(:params) do
        jsonapi_attributes({key: 'test'})
      end

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '201', 'housing channel created' do
        let(:Authorization) { "Bearer #{token_for(responsible)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'housing_channels'
        end
      end
    end
  end

  path '/api/dmo/housing_channels/{id}' do
    put 'edit housing channel' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::HousingChannelsController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
            { key: 'string' }
        )
      end
      let(:housing_channel) { create(:housing_channel) }
      let(:id) { housing_channel.id }

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }
        run_test!
      end 

      response '200', 'housing channel updated' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'housing_channels'
        end
      end
    end
  end

  path '/api/dmo/housing_channels/{id}' do
    delete 'Delete a single housing channel by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      let(:id) { create(:housing_channel, responsible: admin.id).id }

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }
        run_test!
      end 

      response '204', 'housing channel deleted' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        run_test!
      end
    end
  end
end