require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::OperationsController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  include JwtAuthHelper

  check_mutate_params(
    model: Hermes::Operation, 
    mutate_params: Hermes::Api::Dmo::OperationsController.mutate_params, 
    factory: :operation,
    min_params: {
      name: 'test',
    }
  )

  let!(:responsible) { create(:user, roles: ['Administrator']) }
  let!(:user) { create(:user) }
  let!(:lessor) { create(:lessor, siret: '00000000000000', readers: [user.id.to_s]) }
  let!(:lessor2) { create(:lessor, siret: '00000000000000', readers: [responsible.id.to_s]) }
  
  let!(:operation1) { create(:operation, name: 'name1', lessor_id: lessor2.id.to_s) }
  let!(:operation2) { create(:operation, name: 'name2', lessor_id: lessor2.id.to_s) }
  let!(:operation3) { create(:operation, name: 'test3', lessor_id: lessor2.id.to_s) }
  let!(:operation4) { create(:operation, name: 'test4', lessor_id: lessor.id.to_s) }
  

  describe 'POST /operations' do
    describe 'Create as admin' do
      before :each do
        authenticated_header(request, responsible)
        post 'create', {
            data: {
                attributes: {
                    name: 'test'
                }
            }
        }
      end
      it 'returns status code 201' do

        expect(response).to have_http_status(201)
        expect(json['data']['attributes']['code']).to eq('ALSOP-4')
      end
    end

    describe 'Create as user without lessor_id' do
      before :each do
        authenticated_header(request, user)
        post 'create', {
            data: {
                attributes: {
                    name: 'test',
                    code: 'Test'
                }
            }
        }
      end
      it 'returns error when no lessor_id' do
        expect(response).to have_http_status(422)
      end
    end

    describe 'Create as user with lessor_id' do
      before :each do
        authenticated_header(request, user)
        post 'create', {
            data: {
                attributes: {
                    name: 'test',
                    code: 'Bouh',
                    lessor_id: lessor.id.to_s
                }
            }
        }
      end
      it 'returns 201' do
        expect(response).to have_http_status(201)
        expect(json['data']['attributes']['code']).to eq('BOUH')
      end
    end
  end

  describe 'GET /operations' do
    describe 'as admin' do 
      before :each do
        authenticated_header(request, responsible)
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(4)
      end
    end
    
    describe 'filter by lessor_id' do 
      before :each do
        authenticated_header(request, responsible)
        get 'index', {lessor_id: lessor2.id.to_s}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
      end
    end

    describe 'regex on name' do 
      before :each do
        authenticated_header(request, responsible)
        get 'index', {name: {"$regex": "éSt" }}
      end
      it 'returns a valid json' do
        expect(response).to have_http_status(200)
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(2)
      end
    end
    
    describe 'as user' do 
      before :each do
        authenticated_header(request, user)
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(1)
      end
    end
  end

  describe 'GET /operations/:id' do
    describe 'verif it work' do
      before :each do
        authenticated_header(request, user)
        get 'show', {id: operation4.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end

    describe 'verif it work not work with a non authorized user' do

      before :each do
        authenticated_header(request, user)
        get 'show', {id: operation1.id}
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(404)
      end
    end
  end


  describe 'DELETE /operations/:id' do
    before :each do
      authenticated_header(request, responsible)

      delete 'destroy', {
          id: operation1.id.to_s
      }
    end
    it 'returns status code 204' do
      expect(response).to have_http_status(204)
    end
  end

end
require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::OperationsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_OPERATIONS'
  include JwtAuthHelper
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:admin) { create(:user, roles: ['Administrator']) }

  path '/api/dmo/operations' do
    get 'Retrieves all operations' do
      tags tags
      produces 'application/json'

      Hermes::Api::Dmo::OperationsController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'operations found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:operations) { create_list(:operation, 5, name: 'test') }

        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) { operations }
        end
      end
    end
  end

  path '/api/dmo/operations' do
    post 'Create operation' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::OperationsController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {  name: 'test' }
        )
      end

      response '201', 'operation created' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'operations'
        end
      end
    end
  end

  path '/api/dmo/operations/{id}' do
    get 'Retrieves a single operation by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }

        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }

        run_test!
      end 
      
      response '200', 'operation found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:operation) {create(:operation, name: 'test')}
        let(:id) {operation.id}

        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) {operation}
        end
      end
    end
  end

  path '/api/dmo/operations/{id}' do
    put 'Update operation' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::OperationsController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter name: :id, in: :path, type: :string
      let(:id) { create(:operation, name: 'test', user: admin).id }
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {  name: 'test' }
        )
      end

      response '200', 'operation updated' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)
          
          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'operations'
        end
      end
    end
  end

  path '/api/dmo/operations/{id}' do
    delete 'Delete a single operation by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '204', 'operation deleted' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { create(:operation, name: 'test').id }
        run_test!
      end
    end
  end

end
