require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::ReservationHoldersController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  include JwtAuthHelper
  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
  let!(:user) {create(:user)}
  let(:l1) { create(:lessor)}
  let(:l2) { create(:lessor)}
  let(:l3) { create(:lessor)}
  let(:user1) { create(:user, group_ids: [l1.manager_group_id])}
  let(:user2) { create(:user, group_ids: [l2.manager_group_id])}
  let(:user3) { create(:user, group_ids: [l3.manager_group_id])}
  let!(:rh_public) { create(:reservation_holder, name: 'public', public_read: true, siren: ['123456789'])}
  let!(:rh_lessors) { create(:reservation_holder, name: 'lessors', lessors: [l1, l2], siren: ['223456789'])}

  describe 'GET /reservationn_holders/:siren' do
    it 'returns status code 200 with siren' do
      authenticated_header(request, user1)
      get 'show', id: rh_public.siren.first
      expect(response).to have_http_status(200)
      expect(json['data']['id']).to eq(rh_public.id.to_s)
    end

    it 'returns status code 404 with wrong siren' do
      authenticated_header(request, user1)
      get 'show', id: '323456789'
      expect(response).to have_http_status(404)
    end

    it 'returns status code 200 with id' do
      authenticated_header(request, user1)
      get 'show', id: rh_lessors.id
      expect(response).to have_http_status(200)
      expect(json['data']['id']).to eq(rh_lessors.id.to_s)
    end
  end

  describe 'GET /reservationn_holders' do
    it 'returns all for user with access' do
      authenticated_header(request, user1)
      get 'index'
      expect(json['data'].length).to eq(2)
    end

    it 'only return public for user without access' do
      authenticated_header(request, user3)
      get 'index'
      expect(json['data'].length).to eq(1)
    end
  end
end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::ReservationHoldersController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_RESERVATION_HOLDERS'
  include JwtAuthHelper
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:admin) { create(:user, roles: ['Administrator']) }

  path '/api/dmo/reservation_holders' do
    get 'Retrieves all reservation_holders' do
      tags tags
      produces 'application/json'

      Hermes::Api::Dmo::ReservationHoldersController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'reservation_holders found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:reservation_holders) { create_list(:reservation_holder, 1, name: 'test', user: admin) }

        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) { reservation_holders }
        end
      end
    end
  end

  path '/api/dmo/reservation_holders/{id}' do
    get 'Retrieves a single reservation_holder by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }

        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }

        run_test!
      end 
      
      response '200', 'reservation_holder found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:reservation_holder) {create(:reservation_holder, name: 'test', user: admin)}
        let(:id) {reservation_holder.id}

        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) {reservation_holder}
        end
      end
    end
  end

end