require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::SignupUserNotificationPreferencesController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  include JwtAuthHelper
  let!(:responsible) { create(:user, roles: ['Administrator']) }
  let!(:user) { create(:user) }
  

  describe 'POST /signup_user_notification_preferences' do
    let(:email) {'test@email.com'}
    describe 'Create without user' do
      it 'returns status code 201' do
        post 'create', {
            data: {
                attributes: {
                    email: email,
                    notification_key: 'user.new_reserved_offer', 
                    want_to_be_subscribed: false
                }
            }
        }
        expect(response).to have_http_status(201)
        expect(json['data']['attributes']['email']).to eq(email)
      end
      it 'throw error when missign params' do 
        post 'create', {
            data: {
                attributes: {
                    email: email,
                    notification_key: 'user.new_reserved_offer', 
                }
            }
        }
        expect(response).to have_http_status(422)
      end 
    end
  end

  describe 'POST /create_multiple_preferences' do
    let(:email) {'test@email.com'}
    describe 'Create without user' do
      it 'returns status code 201' do
        post 'create_multiple_preferences', {
            data: {
                attributes: {
                    email: email,
                    notifs:[
                      { notification_key: 'user.new_reserved_offer', 
                        want_to_be_subscribed: false},
                        {
                          notification_key: 'user.new_published_offer', 
                          want_to_be_subscribed: false
                        }
                    ]
                    
                }
            }
        }
        expect(response).to have_http_status(201)
        expect(json['email']).to eq(email)
        expect(json['signup_user_notification_preference_ids'].length).to eq(2)
      end
      it 'throw error when missign params' do 
        post 'create_multiple_preferences', {
            data: {
                attributes: {
                    email: email,
                    notifs:[
                      { notification_key: 'user.new_reserved_offer', 
                        want_to_be_subscribed: false},
                        {
                          notification_key: 'user.new_published_offer', 
                        }
                    ]
                }
            }
        }
        expect(response).to have_http_status(422)
      end 
    end
  end

  describe 'GET /signup_user_notification_preferences/user_notifications_list' do
    it 'returns list of user notifications' do
      get 'user_notifications_list'
      expect(response).to have_http_status(200)
      expect(json.length).not_to eq(0)
      expect(json).not_to eq( NOTIFICATION_KEYS[:user].to_json)
    end
  end

end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::SignupUserNotificationPreferencesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_SIGNUP_USER_NOTIFICATION_PREFERENCES'
  include JwtAuthHelper
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:admin) { create(:user, roles: ['Administrator']) }

  path '/api/dmo/signup_user_notification_preferences' do
    post 'Create signup_user_notification_preference' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::SignupUserNotificationPreferencesController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          { 
            email: 'string',
            notification_key: 'string',
            want_to_be_subscribed: true,
            }
        )
      end

      response '201', 'signup_user_notification_preference created' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'signup_user_notification_preferences'
        end
      end
    end
  end

  path '/api/dmo/signup_user_notification_preferences/create_multiple_preferences' do
    post 'Create multiple' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::SignupUserNotificationPreferencesController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(
          email: {type: 'string'},
          notifs: {
            type: 'array',
            items: {
              type: 'object',
              properties: {
                notification_key: {type: 'string'},
                want_to_be_subscribed: {type: 'boolean'}
              }
            }
          }
        ),
      )
      let(:params) do
        jsonapi_attributes(
          { 
            email: 'email',
            notifs:[
              { notification_key: 'user.new_reserved_offer', want_to_be_subscribed: false },
              { notification_key: 'user.new_published_offer', want_to_be_subscribed: false }
            ]
            }
        )
      end

      response '201', 'signup_user_notification_preference created' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        run_test!
      end
    end
  end
end
